import type { Deque } from "./types/Deque";
declare const customInspectSymbol: unique symbol;
/**
 * Deque aka Double-ended queue\
 * O(1) get, insert, and delete for front and back elements
 */
export declare class DequeImpl<T> implements Deque<T> {
    private head;
    private tail;
    private _size;
    /**
     * Initializes Deque with given variables
     *
     * @example
     * ```ts
     * let deque = new Deque(1, 2, 3);
     * deque.toString(); // Deque: [1, 2, 3]
     * deque = new Deque<number>();
     * deque.toString(); // Deque: Deque is empty
     * ```
     * Calling the constructor with an array will create a
     * Deque with a single element which is an array, if you wish
     * to create a Deque with the elements from the array either use
     * the spread operator(new Deque(...arr)) or use Deque.fromArray(arr)
     * @param initData values to initialize the deque with
     */
    constructor(...initData: T[]);
    /**
     * Constructs a Deque from an array of elements
     *
     * @example
     * ```ts
     * const deque = Deque.fromArray([1, 2, 3]);
     * deque.toString(); // Deque: [1, 2, 3]
     * ```
     * @param initData array of elements to create deque with
     */
    static fromArray<T>(initData: T[]): DequeImpl<T>;
    private static createFromArray;
    /**
     * Number of elements in the deque
     */
    get size(): number;
    /**
     * @returns value at front, or undefined if deque is empty
     */
    front(): NonNullable<T> | undefined;
    /**
     * @returns value at back, or undefined if deque is empty
     */
    back(): NonNullable<T> | undefined;
    /**
     * Enqueues value by inserting at front
     *
     * @param data value to insert
     * @returns size after insertion
     */
    enqueue(data: T): number;
    /**
     * Dequeues value by removing from back
     *
     * @returns value at back or undefined if deque is empty
     */
    dequeue(): T | undefined;
    /**
     * Pushes to the stack represented by this deque by inserting at the front
     *
     * @param data value to insert
     * @returns size after insertion
     */
    push(data: T): number;
    /**
     * Pops from the stack represented by this deque by removing from the front
     *
     * @returns value at front or undefined if deque is empty
     */
    pop(): T | undefined;
    /**
     * Inserts value at front
     *
     * @param data value to insert
     * @returns size after insertion
     */
    pushFront(data: T): number;
    /**
     * Inserts value at back
     *
     * @param data value to insert
     * @returns size after insertion
     */
    pushBack(data: T): number;
    /**
     * Removes value from front and returns it, or returns undefined if deque is empty
     *
     * @returns value removed
     */
    popFront(): T | undefined;
    /**
     * Removes value from end and returns it, or returns undefined if deque is empty
     *
     * @returns value removed
     */
    popBack(): T | undefined;
    /**
     * Removes all elements
     */
    clear(): void;
    private getNodeAtIndex;
    private getNodeByValue;
    /**
     * Finds and returns value at given 0-based index
     *
     * @param index of item to retrieve
     * @returns item at index given, or undefined if invalid index given
     */
    get(index: number): T | undefined;
    /**
     * Inserts value at given index, if an out of bounds index is given then inserts
     * at front or back, depending on which direction the index is out of bounds
     *
     * @param index insertion index
     * @param value value to insert
     * @returns new size of the deque
     */
    insert(index: number, value: T): number;
    /**
     * Removes first occurance of a value
     *
     * @param value value to remove
     * @param [isEqualFn=isEqual] optional custom equality function, defaults to Lodash's isEqual
     * @returns value removed, or undefined if value not found
     */
    remove(value: T, isEqualFn?: (val: T, other: T) => boolean): T | undefined;
    /**
     * Counts number of elements that equal to the value given
     *
     * @param value value to search for
     * @param [isEqualFn=isEqual] optional custom equality function, defaults to Lodash's isEqual
     * @returns number of occurances of value in the deque
     */
    count(value: T, isEqualFn?: (val: T, other: T) => boolean): number;
    /**
     * Searches for a value and returns the first found index, or -1 if not found
     *
     * @param value value to search for
     * @param [isEqualFn=isEqual] optional custom equality function, defaults to Lodash's isEqual
     * @returns index of value in the deque or -1 if not found
     */
    indexOf(value: T, isEqualFn?: (val: T, other: T) => boolean): number;
    /**
     * Determines if given value exists in the deque
     *
     * @param value value to search for
     * @returns if the value exists in the deque
     */
    has(value: T): boolean;
    /**
     * Returns a representation of the deque as an array, creates a copy of objects
     *
     * @param [cloneFn=cloneDeep] optional function to use for cloning each node, defaults to Lodash's cloneDeep
     * @returns array of values in order maintained in deque
     */
    toArray(cloneFn?: <T_1>(value: T_1) => T_1): T[];
    /**
     * Reverses the deque in-place
     */
    reverse(): this;
    /**
     * Creates a copy of the deque maintaining order of values, creates a deep copy of objects
     *
     * @param [cloneFn=cloneDeep] optional function to use for cloning each node, defaults to lodash's cloneDeep
     * @returns a new deque instance with copied values
     */
    copy(cloneFn?: <T_1>(value: T_1) => T_1): DequeImpl<T>;
    toString(): string;
    [customInspectSymbol](): string;
}
export {};
